""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: vpn/ipsec/phase1
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class Phase1CertificateItem:
    """Nested item for certificate field - supports attribute access."""
    name: str


class Phase1InternaldomainlistItem:
    """Nested item for internal-domain-list field - supports attribute access."""
    domain_name: str


class Phase1DnssuffixsearchItem:
    """Nested item for dns-suffix-search field - supports attribute access."""
    dns_suffix: str


class Phase1Ipv4excluderangeItem:
    """Nested item for ipv4-exclude-range field - supports attribute access."""
    id: int
    start_ip: str
    end_ip: str


class Phase1Ipv6excluderangeItem:
    """Nested item for ipv6-exclude-range field - supports attribute access."""
    id: int
    start_ip: str
    end_ip: str


class Phase1BackupgatewayItem:
    """Nested item for backup-gateway field - supports attribute access."""
    address: str


class Phase1RemotegwztnatagsItem:
    """Nested item for remote-gw-ztna-tags field - supports attribute access."""
    name: str


class Phase1Payload(TypedDict, total=False):
    """Payload type for Phase1 operations."""
    name: str
    type: Literal["static", "dynamic", "ddns"]
    interface: str
    ike_version: Literal["1", "2"]
    remote_gw: str
    local_gw: str
    remotegw_ddns: str
    keylife: int
    certificate: str | list[str] | list[dict[str, Any]] | list[Phase1CertificateItem]
    authmethod: Literal["psk", "signature"]
    authmethod_remote: Literal["psk", "signature"]
    mode: Literal["aggressive", "main"]
    peertype: Literal["any", "one", "dialup", "peer", "peergrp"]
    peerid: str
    usrgrp: str
    peer: str
    peergrp: str
    mode_cfg: Literal["disable", "enable"]
    mode_cfg_allow_client_selector: Literal["disable", "enable"]
    assign_ip: Literal["disable", "enable"]
    assign_ip_from: Literal["range", "usrgrp", "dhcp", "name"]
    ipv4_start_ip: str
    ipv4_end_ip: str
    ipv4_netmask: str
    dhcp_ra_giaddr: str
    dhcp6_ra_linkaddr: str
    dns_mode: Literal["manual", "auto"]
    ipv4_dns_server1: str
    ipv4_dns_server2: str
    ipv4_dns_server3: str
    internal_domain_list: str | list[str] | list[dict[str, Any]] | list[Phase1InternaldomainlistItem]
    dns_suffix_search: str | list[str] | list[dict[str, Any]] | list[Phase1DnssuffixsearchItem]
    ipv4_wins_server1: str
    ipv4_wins_server2: str
    ipv4_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv4excluderangeItem]
    ipv4_split_include: str
    split_include_service: str
    ipv4_name: str
    ipv6_start_ip: str
    ipv6_end_ip: str
    ipv6_prefix: int
    ipv6_dns_server1: str
    ipv6_dns_server2: str
    ipv6_dns_server3: str
    ipv6_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv6excluderangeItem]
    ipv6_split_include: str
    ipv6_name: str
    ip_delay_interval: int
    unity_support: Literal["disable", "enable"]
    domain: str
    banner: str
    include_local_lan: Literal["disable", "enable"]
    ipv4_split_exclude: str
    ipv6_split_exclude: str
    save_password: Literal["disable", "enable"]
    client_auto_negotiate: Literal["disable", "enable"]
    client_keep_alive: Literal["disable", "enable"]
    backup_gateway: str | list[str] | list[dict[str, Any]] | list[Phase1BackupgatewayItem]
    proposal: str | list[str]
    add_route: Literal["disable", "enable"]
    add_gw_route: Literal["enable", "disable"]
    psksecret: str
    psksecret_remote: str
    keepalive: int
    distance: int
    priority: int
    localid: str
    localid_type: Literal["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"]
    auto_negotiate: Literal["enable", "disable"]
    negotiate_timeout: int
    fragmentation: Literal["enable", "disable"]
    dpd: Literal["disable", "on-idle", "on-demand"]
    dpd_retrycount: int
    dpd_retryinterval: str
    comments: str
    npu_offload: Literal["enable", "disable"]
    send_cert_chain: Literal["enable", "disable"]
    dhgrp: str | list[str]
    addke1: str | list[str]
    addke2: str | list[str]
    addke3: str | list[str]
    addke4: str | list[str]
    addke5: str | list[str]
    addke6: str | list[str]
    addke7: str | list[str]
    suite_b: Literal["disable", "suite-b-gcm-128", "suite-b-gcm-256"]
    eap: Literal["enable", "disable"]
    eap_identity: Literal["use-id-payload", "send-request"]
    eap_exclude_peergrp: str
    eap_cert_auth: Literal["enable", "disable"]
    acct_verify: Literal["enable", "disable"]
    ppk: Literal["disable", "allow", "require"]
    ppk_secret: str
    ppk_identity: str
    wizard_type: Literal["custom", "dialup-forticlient", "dialup-ios", "dialup-android", "dialup-windows", "dialup-cisco", "static-fortigate", "dialup-fortigate", "static-cisco", "dialup-cisco-fw", "simplified-static-fortigate", "hub-fortigate-auto-discovery", "spoke-fortigate-auto-discovery", "fabric-overlay-orchestrator"]
    xauthtype: Literal["disable", "client", "pap", "chap", "auto"]
    reauth: Literal["disable", "enable"]
    authusr: str
    authpasswd: str
    group_authentication: Literal["enable", "disable"]
    group_authentication_secret: str
    authusrgrp: str
    mesh_selector_type: Literal["disable", "subnet", "host"]
    idle_timeout: Literal["enable", "disable"]
    shared_idle_timeout: Literal["enable", "disable"]
    idle_timeoutinterval: int
    ha_sync_esp_seqno: Literal["enable", "disable"]
    fgsp_sync: Literal["enable", "disable"]
    inbound_dscp_copy: Literal["enable", "disable"]
    nattraversal: Literal["enable", "disable", "forced"]
    esn: Literal["require", "allow", "disable"]
    fragmentation_mtu: int
    childless_ike: Literal["enable", "disable"]
    azure_ad_autoconnect: Literal["enable", "disable"]
    client_resume: Literal["enable", "disable"]
    client_resume_interval: int
    rekey: Literal["enable", "disable"]
    digital_signature_auth: Literal["enable", "disable"]
    signature_hash_alg: str | list[str]
    rsa_signature_format: Literal["pkcs1", "pss"]
    rsa_signature_hash_override: Literal["enable", "disable"]
    enforce_unique_id: Literal["disable", "keep-new", "keep-old"]
    cert_id_validation: Literal["enable", "disable"]
    fec_egress: Literal["enable", "disable"]
    fec_send_timeout: int
    fec_base: int
    fec_codec: Literal["rs", "xor"]
    fec_redundant: int
    fec_ingress: Literal["enable", "disable"]
    fec_receive_timeout: int
    fec_health_check: str
    fec_mapping_profile: str
    network_overlay: Literal["disable", "enable"]
    network_id: int
    dev_id_notification: Literal["disable", "enable"]
    dev_id: str
    loopback_asymroute: Literal["enable", "disable"]
    link_cost: int
    kms: str
    exchange_fgt_device_id: Literal["enable", "disable"]
    ipv6_auto_linklocal: Literal["enable", "disable"]
    ems_sn_check: Literal["enable", "disable"]
    cert_trust_store: Literal["local", "ems"]
    qkd: Literal["disable", "allow", "require"]
    qkd_hybrid: Literal["disable", "allow", "require"]
    qkd_profile: str
    transport: Literal["udp", "auto", "tcp"]
    fortinet_esp: Literal["enable", "disable"]
    auto_transport_threshold: int
    remote_gw_match: Literal["any", "ipmask", "iprange", "geography", "ztna"]
    remote_gw_subnet: str
    remote_gw_start_ip: str
    remote_gw_end_ip: str
    remote_gw_country: str
    remote_gw_ztna_tags: str | list[str] | list[dict[str, Any]] | list[Phase1RemotegwztnatagsItem]
    remote_gw6_match: Literal["any", "ipprefix", "iprange", "geography"]
    remote_gw6_subnet: str
    remote_gw6_start_ip: str
    remote_gw6_end_ip: str
    remote_gw6_country: str
    cert_peer_username_validation: Literal["none", "othername", "rfc822name", "cn"]
    cert_peer_username_strip: Literal["disable", "enable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class Phase1Response(TypedDict, total=False):
    """Response type for Phase1 - use with .dict property for typed dict access."""
    name: str
    type: Literal["static", "dynamic", "ddns"]
    interface: str
    ike_version: Literal["1", "2"]
    remote_gw: str
    local_gw: str
    remotegw_ddns: str
    keylife: int
    certificate: list[Phase1CertificateItem]
    authmethod: Literal["psk", "signature"]
    authmethod_remote: Literal["psk", "signature"]
    mode: Literal["aggressive", "main"]
    peertype: Literal["any", "one", "dialup", "peer", "peergrp"]
    peerid: str
    usrgrp: str
    peer: str
    peergrp: str
    mode_cfg: Literal["disable", "enable"]
    mode_cfg_allow_client_selector: Literal["disable", "enable"]
    assign_ip: Literal["disable", "enable"]
    assign_ip_from: Literal["range", "usrgrp", "dhcp", "name"]
    ipv4_start_ip: str
    ipv4_end_ip: str
    ipv4_netmask: str
    dhcp_ra_giaddr: str
    dhcp6_ra_linkaddr: str
    dns_mode: Literal["manual", "auto"]
    ipv4_dns_server1: str
    ipv4_dns_server2: str
    ipv4_dns_server3: str
    internal_domain_list: list[Phase1InternaldomainlistItem]
    dns_suffix_search: list[Phase1DnssuffixsearchItem]
    ipv4_wins_server1: str
    ipv4_wins_server2: str
    ipv4_exclude_range: list[Phase1Ipv4excluderangeItem]
    ipv4_split_include: str
    split_include_service: str
    ipv4_name: str
    ipv6_start_ip: str
    ipv6_end_ip: str
    ipv6_prefix: int
    ipv6_dns_server1: str
    ipv6_dns_server2: str
    ipv6_dns_server3: str
    ipv6_exclude_range: list[Phase1Ipv6excluderangeItem]
    ipv6_split_include: str
    ipv6_name: str
    ip_delay_interval: int
    unity_support: Literal["disable", "enable"]
    domain: str
    banner: str
    include_local_lan: Literal["disable", "enable"]
    ipv4_split_exclude: str
    ipv6_split_exclude: str
    save_password: Literal["disable", "enable"]
    client_auto_negotiate: Literal["disable", "enable"]
    client_keep_alive: Literal["disable", "enable"]
    backup_gateway: list[Phase1BackupgatewayItem]
    proposal: str
    add_route: Literal["disable", "enable"]
    add_gw_route: Literal["enable", "disable"]
    psksecret: str
    psksecret_remote: str
    keepalive: int
    distance: int
    priority: int
    localid: str
    localid_type: Literal["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"]
    auto_negotiate: Literal["enable", "disable"]
    negotiate_timeout: int
    fragmentation: Literal["enable", "disable"]
    dpd: Literal["disable", "on-idle", "on-demand"]
    dpd_retrycount: int
    dpd_retryinterval: str
    comments: str
    npu_offload: Literal["enable", "disable"]
    send_cert_chain: Literal["enable", "disable"]
    dhgrp: str
    addke1: str
    addke2: str
    addke3: str
    addke4: str
    addke5: str
    addke6: str
    addke7: str
    suite_b: Literal["disable", "suite-b-gcm-128", "suite-b-gcm-256"]
    eap: Literal["enable", "disable"]
    eap_identity: Literal["use-id-payload", "send-request"]
    eap_exclude_peergrp: str
    eap_cert_auth: Literal["enable", "disable"]
    acct_verify: Literal["enable", "disable"]
    ppk: Literal["disable", "allow", "require"]
    ppk_secret: str
    ppk_identity: str
    wizard_type: Literal["custom", "dialup-forticlient", "dialup-ios", "dialup-android", "dialup-windows", "dialup-cisco", "static-fortigate", "dialup-fortigate", "static-cisco", "dialup-cisco-fw", "simplified-static-fortigate", "hub-fortigate-auto-discovery", "spoke-fortigate-auto-discovery", "fabric-overlay-orchestrator"]
    xauthtype: Literal["disable", "client", "pap", "chap", "auto"]
    reauth: Literal["disable", "enable"]
    authusr: str
    authpasswd: str
    group_authentication: Literal["enable", "disable"]
    group_authentication_secret: str
    authusrgrp: str
    mesh_selector_type: Literal["disable", "subnet", "host"]
    idle_timeout: Literal["enable", "disable"]
    shared_idle_timeout: Literal["enable", "disable"]
    idle_timeoutinterval: int
    ha_sync_esp_seqno: Literal["enable", "disable"]
    fgsp_sync: Literal["enable", "disable"]
    inbound_dscp_copy: Literal["enable", "disable"]
    nattraversal: Literal["enable", "disable", "forced"]
    esn: Literal["require", "allow", "disable"]
    fragmentation_mtu: int
    childless_ike: Literal["enable", "disable"]
    azure_ad_autoconnect: Literal["enable", "disable"]
    client_resume: Literal["enable", "disable"]
    client_resume_interval: int
    rekey: Literal["enable", "disable"]
    digital_signature_auth: Literal["enable", "disable"]
    signature_hash_alg: str
    rsa_signature_format: Literal["pkcs1", "pss"]
    rsa_signature_hash_override: Literal["enable", "disable"]
    enforce_unique_id: Literal["disable", "keep-new", "keep-old"]
    cert_id_validation: Literal["enable", "disable"]
    fec_egress: Literal["enable", "disable"]
    fec_send_timeout: int
    fec_base: int
    fec_codec: Literal["rs", "xor"]
    fec_redundant: int
    fec_ingress: Literal["enable", "disable"]
    fec_receive_timeout: int
    fec_health_check: str
    fec_mapping_profile: str
    network_overlay: Literal["disable", "enable"]
    network_id: int
    dev_id_notification: Literal["disable", "enable"]
    dev_id: str
    loopback_asymroute: Literal["enable", "disable"]
    link_cost: int
    kms: str
    exchange_fgt_device_id: Literal["enable", "disable"]
    ipv6_auto_linklocal: Literal["enable", "disable"]
    ems_sn_check: Literal["enable", "disable"]
    cert_trust_store: Literal["local", "ems"]
    qkd: Literal["disable", "allow", "require"]
    qkd_hybrid: Literal["disable", "allow", "require"]
    qkd_profile: str
    transport: Literal["udp", "auto", "tcp"]
    fortinet_esp: Literal["enable", "disable"]
    auto_transport_threshold: int
    remote_gw_match: Literal["any", "ipmask", "iprange", "geography", "ztna"]
    remote_gw_subnet: str
    remote_gw_start_ip: str
    remote_gw_end_ip: str
    remote_gw_country: str
    remote_gw_ztna_tags: list[Phase1RemotegwztnatagsItem]
    remote_gw6_match: Literal["any", "ipprefix", "iprange", "geography"]
    remote_gw6_subnet: str
    remote_gw6_start_ip: str
    remote_gw6_end_ip: str
    remote_gw6_country: str
    cert_peer_username_validation: Literal["none", "othername", "rfc822name", "cn"]
    cert_peer_username_strip: Literal["disable", "enable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class Phase1Object(FortiObject):
    """Typed FortiObject for Phase1 with field access."""
    name: str
    type: Literal["static", "dynamic", "ddns"]
    interface: str
    ike_version: Literal["1", "2"]
    remote_gw: str
    local_gw: str
    remotegw_ddns: str
    keylife: int
    certificate: list[Phase1CertificateItem]
    authmethod: Literal["psk", "signature"]
    authmethod_remote: Literal["psk", "signature"]
    mode: Literal["aggressive", "main"]
    peertype: Literal["any", "one", "dialup", "peer", "peergrp"]
    peerid: str
    usrgrp: str
    peer: str
    peergrp: str
    mode_cfg: Literal["disable", "enable"]
    mode_cfg_allow_client_selector: Literal["disable", "enable"]
    assign_ip: Literal["disable", "enable"]
    assign_ip_from: Literal["range", "usrgrp", "dhcp", "name"]
    ipv4_start_ip: str
    ipv4_end_ip: str
    ipv4_netmask: str
    dhcp_ra_giaddr: str
    dhcp6_ra_linkaddr: str
    dns_mode: Literal["manual", "auto"]
    ipv4_dns_server1: str
    ipv4_dns_server2: str
    ipv4_dns_server3: str
    internal_domain_list: list[Phase1InternaldomainlistItem]
    dns_suffix_search: list[Phase1DnssuffixsearchItem]
    ipv4_wins_server1: str
    ipv4_wins_server2: str
    ipv4_exclude_range: list[Phase1Ipv4excluderangeItem]
    ipv4_split_include: str
    split_include_service: str
    ipv4_name: str
    ipv6_start_ip: str
    ipv6_end_ip: str
    ipv6_prefix: int
    ipv6_dns_server1: str
    ipv6_dns_server2: str
    ipv6_dns_server3: str
    ipv6_exclude_range: list[Phase1Ipv6excluderangeItem]
    ipv6_split_include: str
    ipv6_name: str
    ip_delay_interval: int
    unity_support: Literal["disable", "enable"]
    domain: str
    banner: str
    include_local_lan: Literal["disable", "enable"]
    ipv4_split_exclude: str
    ipv6_split_exclude: str
    save_password: Literal["disable", "enable"]
    client_auto_negotiate: Literal["disable", "enable"]
    client_keep_alive: Literal["disable", "enable"]
    backup_gateway: list[Phase1BackupgatewayItem]
    proposal: str
    add_route: Literal["disable", "enable"]
    add_gw_route: Literal["enable", "disable"]
    psksecret: str
    psksecret_remote: str
    keepalive: int
    distance: int
    priority: int
    localid: str
    localid_type: Literal["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"]
    auto_negotiate: Literal["enable", "disable"]
    negotiate_timeout: int
    fragmentation: Literal["enable", "disable"]
    dpd: Literal["disable", "on-idle", "on-demand"]
    dpd_retrycount: int
    dpd_retryinterval: str
    comments: str
    npu_offload: Literal["enable", "disable"]
    send_cert_chain: Literal["enable", "disable"]
    dhgrp: str
    addke1: str
    addke2: str
    addke3: str
    addke4: str
    addke5: str
    addke6: str
    addke7: str
    suite_b: Literal["disable", "suite-b-gcm-128", "suite-b-gcm-256"]
    eap: Literal["enable", "disable"]
    eap_identity: Literal["use-id-payload", "send-request"]
    eap_exclude_peergrp: str
    eap_cert_auth: Literal["enable", "disable"]
    acct_verify: Literal["enable", "disable"]
    ppk: Literal["disable", "allow", "require"]
    ppk_secret: str
    ppk_identity: str
    wizard_type: Literal["custom", "dialup-forticlient", "dialup-ios", "dialup-android", "dialup-windows", "dialup-cisco", "static-fortigate", "dialup-fortigate", "static-cisco", "dialup-cisco-fw", "simplified-static-fortigate", "hub-fortigate-auto-discovery", "spoke-fortigate-auto-discovery", "fabric-overlay-orchestrator"]
    xauthtype: Literal["disable", "client", "pap", "chap", "auto"]
    reauth: Literal["disable", "enable"]
    authusr: str
    authpasswd: str
    group_authentication: Literal["enable", "disable"]
    group_authentication_secret: str
    authusrgrp: str
    mesh_selector_type: Literal["disable", "subnet", "host"]
    idle_timeout: Literal["enable", "disable"]
    shared_idle_timeout: Literal["enable", "disable"]
    idle_timeoutinterval: int
    ha_sync_esp_seqno: Literal["enable", "disable"]
    fgsp_sync: Literal["enable", "disable"]
    inbound_dscp_copy: Literal["enable", "disable"]
    nattraversal: Literal["enable", "disable", "forced"]
    esn: Literal["require", "allow", "disable"]
    fragmentation_mtu: int
    childless_ike: Literal["enable", "disable"]
    azure_ad_autoconnect: Literal["enable", "disable"]
    client_resume: Literal["enable", "disable"]
    client_resume_interval: int
    rekey: Literal["enable", "disable"]
    digital_signature_auth: Literal["enable", "disable"]
    signature_hash_alg: str
    rsa_signature_format: Literal["pkcs1", "pss"]
    rsa_signature_hash_override: Literal["enable", "disable"]
    enforce_unique_id: Literal["disable", "keep-new", "keep-old"]
    cert_id_validation: Literal["enable", "disable"]
    fec_egress: Literal["enable", "disable"]
    fec_send_timeout: int
    fec_base: int
    fec_codec: Literal["rs", "xor"]
    fec_redundant: int
    fec_ingress: Literal["enable", "disable"]
    fec_receive_timeout: int
    fec_health_check: str
    fec_mapping_profile: str
    network_overlay: Literal["disable", "enable"]
    network_id: int
    dev_id_notification: Literal["disable", "enable"]
    dev_id: str
    loopback_asymroute: Literal["enable", "disable"]
    link_cost: int
    kms: str
    exchange_fgt_device_id: Literal["enable", "disable"]
    ipv6_auto_linklocal: Literal["enable", "disable"]
    ems_sn_check: Literal["enable", "disable"]
    cert_trust_store: Literal["local", "ems"]
    qkd: Literal["disable", "allow", "require"]
    qkd_hybrid: Literal["disable", "allow", "require"]
    qkd_profile: str
    transport: Literal["udp", "auto", "tcp"]
    fortinet_esp: Literal["enable", "disable"]
    auto_transport_threshold: int
    remote_gw_match: Literal["any", "ipmask", "iprange", "geography", "ztna"]
    remote_gw_subnet: str
    remote_gw_start_ip: str
    remote_gw_end_ip: str
    remote_gw_country: str
    remote_gw_ztna_tags: list[Phase1RemotegwztnatagsItem]
    remote_gw6_match: Literal["any", "ipprefix", "iprange", "geography"]
    remote_gw6_subnet: str
    remote_gw6_start_ip: str
    remote_gw6_end_ip: str
    remote_gw6_country: str
    cert_peer_username_validation: Literal["none", "othername", "rfc822name", "cn"]
    cert_peer_username_strip: Literal["disable", "enable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Phase1:
    """
    
    Endpoint: vpn/ipsec/phase1
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Phase1Object: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[Phase1Object]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: Phase1Payload | None = ...,
        name: str | None = ...,
        type: Literal["static", "dynamic", "ddns"] | None = ...,
        interface: str | None = ...,
        ike_version: Literal["1", "2"] | None = ...,
        remote_gw: str | None = ...,
        local_gw: str | None = ...,
        remotegw_ddns: str | None = ...,
        keylife: int | None = ...,
        certificate: str | list[str] | list[dict[str, Any]] | list[Phase1CertificateItem] | None = ...,
        authmethod: Literal["psk", "signature"] | None = ...,
        authmethod_remote: Literal["psk", "signature"] | None = ...,
        mode: Literal["aggressive", "main"] | None = ...,
        peertype: Literal["any", "one", "dialup", "peer", "peergrp"] | None = ...,
        peerid: str | None = ...,
        usrgrp: str | None = ...,
        peer: str | None = ...,
        peergrp: str | None = ...,
        mode_cfg: Literal["disable", "enable"] | None = ...,
        mode_cfg_allow_client_selector: Literal["disable", "enable"] | None = ...,
        assign_ip: Literal["disable", "enable"] | None = ...,
        assign_ip_from: Literal["range", "usrgrp", "dhcp", "name"] | None = ...,
        ipv4_start_ip: str | None = ...,
        ipv4_end_ip: str | None = ...,
        ipv4_netmask: str | None = ...,
        dhcp_ra_giaddr: str | None = ...,
        dhcp6_ra_linkaddr: str | None = ...,
        dns_mode: Literal["manual", "auto"] | None = ...,
        ipv4_dns_server1: str | None = ...,
        ipv4_dns_server2: str | None = ...,
        ipv4_dns_server3: str | None = ...,
        internal_domain_list: str | list[str] | list[dict[str, Any]] | list[Phase1InternaldomainlistItem] | None = ...,
        dns_suffix_search: str | list[str] | list[dict[str, Any]] | list[Phase1DnssuffixsearchItem] | None = ...,
        ipv4_wins_server1: str | None = ...,
        ipv4_wins_server2: str | None = ...,
        ipv4_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv4excluderangeItem] | None = ...,
        ipv4_split_include: str | None = ...,
        split_include_service: str | None = ...,
        ipv4_name: str | None = ...,
        ipv6_start_ip: str | None = ...,
        ipv6_end_ip: str | None = ...,
        ipv6_prefix: int | None = ...,
        ipv6_dns_server1: str | None = ...,
        ipv6_dns_server2: str | None = ...,
        ipv6_dns_server3: str | None = ...,
        ipv6_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv6excluderangeItem] | None = ...,
        ipv6_split_include: str | None = ...,
        ipv6_name: str | None = ...,
        ip_delay_interval: int | None = ...,
        unity_support: Literal["disable", "enable"] | None = ...,
        domain: str | None = ...,
        banner: str | None = ...,
        include_local_lan: Literal["disable", "enable"] | None = ...,
        ipv4_split_exclude: str | None = ...,
        ipv6_split_exclude: str | None = ...,
        save_password: Literal["disable", "enable"] | None = ...,
        client_auto_negotiate: Literal["disable", "enable"] | None = ...,
        client_keep_alive: Literal["disable", "enable"] | None = ...,
        backup_gateway: str | list[str] | list[dict[str, Any]] | list[Phase1BackupgatewayItem] | None = ...,
        proposal: str | list[str] | None = ...,
        add_route: Literal["disable", "enable"] | None = ...,
        add_gw_route: Literal["enable", "disable"] | None = ...,
        psksecret: str | None = ...,
        psksecret_remote: str | None = ...,
        keepalive: int | None = ...,
        distance: int | None = ...,
        priority: int | None = ...,
        localid: str | None = ...,
        localid_type: Literal["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"] | None = ...,
        auto_negotiate: Literal["enable", "disable"] | None = ...,
        negotiate_timeout: int | None = ...,
        fragmentation: Literal["enable", "disable"] | None = ...,
        dpd: Literal["disable", "on-idle", "on-demand"] | None = ...,
        dpd_retrycount: int | None = ...,
        dpd_retryinterval: str | None = ...,
        comments: str | None = ...,
        npu_offload: Literal["enable", "disable"] | None = ...,
        send_cert_chain: Literal["enable", "disable"] | None = ...,
        dhgrp: str | list[str] | None = ...,
        addke1: str | list[str] | None = ...,
        addke2: str | list[str] | None = ...,
        addke3: str | list[str] | None = ...,
        addke4: str | list[str] | None = ...,
        addke5: str | list[str] | None = ...,
        addke6: str | list[str] | None = ...,
        addke7: str | list[str] | None = ...,
        suite_b: Literal["disable", "suite-b-gcm-128", "suite-b-gcm-256"] | None = ...,
        eap: Literal["enable", "disable"] | None = ...,
        eap_identity: Literal["use-id-payload", "send-request"] | None = ...,
        eap_exclude_peergrp: str | None = ...,
        eap_cert_auth: Literal["enable", "disable"] | None = ...,
        acct_verify: Literal["enable", "disable"] | None = ...,
        ppk: Literal["disable", "allow", "require"] | None = ...,
        ppk_secret: str | None = ...,
        ppk_identity: str | None = ...,
        wizard_type: Literal["custom", "dialup-forticlient", "dialup-ios", "dialup-android", "dialup-windows", "dialup-cisco", "static-fortigate", "dialup-fortigate", "static-cisco", "dialup-cisco-fw", "simplified-static-fortigate", "hub-fortigate-auto-discovery", "spoke-fortigate-auto-discovery", "fabric-overlay-orchestrator"] | None = ...,
        xauthtype: Literal["disable", "client", "pap", "chap", "auto"] | None = ...,
        reauth: Literal["disable", "enable"] | None = ...,
        authusr: str | None = ...,
        authpasswd: str | None = ...,
        group_authentication: Literal["enable", "disable"] | None = ...,
        group_authentication_secret: str | None = ...,
        authusrgrp: str | None = ...,
        mesh_selector_type: Literal["disable", "subnet", "host"] | None = ...,
        idle_timeout: Literal["enable", "disable"] | None = ...,
        shared_idle_timeout: Literal["enable", "disable"] | None = ...,
        idle_timeoutinterval: int | None = ...,
        ha_sync_esp_seqno: Literal["enable", "disable"] | None = ...,
        fgsp_sync: Literal["enable", "disable"] | None = ...,
        inbound_dscp_copy: Literal["enable", "disable"] | None = ...,
        nattraversal: Literal["enable", "disable", "forced"] | None = ...,
        esn: Literal["require", "allow", "disable"] | None = ...,
        fragmentation_mtu: int | None = ...,
        childless_ike: Literal["enable", "disable"] | None = ...,
        azure_ad_autoconnect: Literal["enable", "disable"] | None = ...,
        client_resume: Literal["enable", "disable"] | None = ...,
        client_resume_interval: int | None = ...,
        rekey: Literal["enable", "disable"] | None = ...,
        digital_signature_auth: Literal["enable", "disable"] | None = ...,
        signature_hash_alg: str | list[str] | None = ...,
        rsa_signature_format: Literal["pkcs1", "pss"] | None = ...,
        rsa_signature_hash_override: Literal["enable", "disable"] | None = ...,
        enforce_unique_id: Literal["disable", "keep-new", "keep-old"] | None = ...,
        cert_id_validation: Literal["enable", "disable"] | None = ...,
        fec_egress: Literal["enable", "disable"] | None = ...,
        fec_send_timeout: int | None = ...,
        fec_base: int | None = ...,
        fec_codec: Literal["rs", "xor"] | None = ...,
        fec_redundant: int | None = ...,
        fec_ingress: Literal["enable", "disable"] | None = ...,
        fec_receive_timeout: int | None = ...,
        fec_health_check: str | None = ...,
        fec_mapping_profile: str | None = ...,
        network_overlay: Literal["disable", "enable"] | None = ...,
        network_id: int | None = ...,
        dev_id_notification: Literal["disable", "enable"] | None = ...,
        dev_id: str | None = ...,
        loopback_asymroute: Literal["enable", "disable"] | None = ...,
        link_cost: int | None = ...,
        kms: str | None = ...,
        exchange_fgt_device_id: Literal["enable", "disable"] | None = ...,
        ipv6_auto_linklocal: Literal["enable", "disable"] | None = ...,
        ems_sn_check: Literal["enable", "disable"] | None = ...,
        cert_trust_store: Literal["local", "ems"] | None = ...,
        qkd: Literal["disable", "allow", "require"] | None = ...,
        qkd_hybrid: Literal["disable", "allow", "require"] | None = ...,
        qkd_profile: str | None = ...,
        transport: Literal["udp", "auto", "tcp"] | None = ...,
        fortinet_esp: Literal["enable", "disable"] | None = ...,
        auto_transport_threshold: int | None = ...,
        remote_gw_match: Literal["any", "ipmask", "iprange", "geography", "ztna"] | None = ...,
        remote_gw_subnet: str | None = ...,
        remote_gw_start_ip: str | None = ...,
        remote_gw_end_ip: str | None = ...,
        remote_gw_country: str | None = ...,
        remote_gw_ztna_tags: str | list[str] | list[dict[str, Any]] | list[Phase1RemotegwztnatagsItem] | None = ...,
        remote_gw6_match: Literal["any", "ipprefix", "iprange", "geography"] | None = ...,
        remote_gw6_subnet: str | None = ...,
        remote_gw6_start_ip: str | None = ...,
        remote_gw6_end_ip: str | None = ...,
        remote_gw6_country: str | None = ...,
        cert_peer_username_validation: Literal["none", "othername", "rfc822name", "cn"] | None = ...,
        cert_peer_username_strip: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Phase1Object: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: Phase1Payload | None = ...,
        name: str | None = ...,
        type: Literal["static", "dynamic", "ddns"] | None = ...,
        interface: str | None = ...,
        ike_version: Literal["1", "2"] | None = ...,
        remote_gw: str | None = ...,
        local_gw: str | None = ...,
        remotegw_ddns: str | None = ...,
        keylife: int | None = ...,
        certificate: str | list[str] | list[dict[str, Any]] | list[Phase1CertificateItem] | None = ...,
        authmethod: Literal["psk", "signature"] | None = ...,
        authmethod_remote: Literal["psk", "signature"] | None = ...,
        mode: Literal["aggressive", "main"] | None = ...,
        peertype: Literal["any", "one", "dialup", "peer", "peergrp"] | None = ...,
        peerid: str | None = ...,
        usrgrp: str | None = ...,
        peer: str | None = ...,
        peergrp: str | None = ...,
        mode_cfg: Literal["disable", "enable"] | None = ...,
        mode_cfg_allow_client_selector: Literal["disable", "enable"] | None = ...,
        assign_ip: Literal["disable", "enable"] | None = ...,
        assign_ip_from: Literal["range", "usrgrp", "dhcp", "name"] | None = ...,
        ipv4_start_ip: str | None = ...,
        ipv4_end_ip: str | None = ...,
        ipv4_netmask: str | None = ...,
        dhcp_ra_giaddr: str | None = ...,
        dhcp6_ra_linkaddr: str | None = ...,
        dns_mode: Literal["manual", "auto"] | None = ...,
        ipv4_dns_server1: str | None = ...,
        ipv4_dns_server2: str | None = ...,
        ipv4_dns_server3: str | None = ...,
        internal_domain_list: str | list[str] | list[dict[str, Any]] | list[Phase1InternaldomainlistItem] | None = ...,
        dns_suffix_search: str | list[str] | list[dict[str, Any]] | list[Phase1DnssuffixsearchItem] | None = ...,
        ipv4_wins_server1: str | None = ...,
        ipv4_wins_server2: str | None = ...,
        ipv4_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv4excluderangeItem] | None = ...,
        ipv4_split_include: str | None = ...,
        split_include_service: str | None = ...,
        ipv4_name: str | None = ...,
        ipv6_start_ip: str | None = ...,
        ipv6_end_ip: str | None = ...,
        ipv6_prefix: int | None = ...,
        ipv6_dns_server1: str | None = ...,
        ipv6_dns_server2: str | None = ...,
        ipv6_dns_server3: str | None = ...,
        ipv6_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv6excluderangeItem] | None = ...,
        ipv6_split_include: str | None = ...,
        ipv6_name: str | None = ...,
        ip_delay_interval: int | None = ...,
        unity_support: Literal["disable", "enable"] | None = ...,
        domain: str | None = ...,
        banner: str | None = ...,
        include_local_lan: Literal["disable", "enable"] | None = ...,
        ipv4_split_exclude: str | None = ...,
        ipv6_split_exclude: str | None = ...,
        save_password: Literal["disable", "enable"] | None = ...,
        client_auto_negotiate: Literal["disable", "enable"] | None = ...,
        client_keep_alive: Literal["disable", "enable"] | None = ...,
        backup_gateway: str | list[str] | list[dict[str, Any]] | list[Phase1BackupgatewayItem] | None = ...,
        proposal: str | list[str] | None = ...,
        add_route: Literal["disable", "enable"] | None = ...,
        add_gw_route: Literal["enable", "disable"] | None = ...,
        psksecret: str | None = ...,
        psksecret_remote: str | None = ...,
        keepalive: int | None = ...,
        distance: int | None = ...,
        priority: int | None = ...,
        localid: str | None = ...,
        localid_type: Literal["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"] | None = ...,
        auto_negotiate: Literal["enable", "disable"] | None = ...,
        negotiate_timeout: int | None = ...,
        fragmentation: Literal["enable", "disable"] | None = ...,
        dpd: Literal["disable", "on-idle", "on-demand"] | None = ...,
        dpd_retrycount: int | None = ...,
        dpd_retryinterval: str | None = ...,
        comments: str | None = ...,
        npu_offload: Literal["enable", "disable"] | None = ...,
        send_cert_chain: Literal["enable", "disable"] | None = ...,
        dhgrp: str | list[str] | None = ...,
        addke1: str | list[str] | None = ...,
        addke2: str | list[str] | None = ...,
        addke3: str | list[str] | None = ...,
        addke4: str | list[str] | None = ...,
        addke5: str | list[str] | None = ...,
        addke6: str | list[str] | None = ...,
        addke7: str | list[str] | None = ...,
        suite_b: Literal["disable", "suite-b-gcm-128", "suite-b-gcm-256"] | None = ...,
        eap: Literal["enable", "disable"] | None = ...,
        eap_identity: Literal["use-id-payload", "send-request"] | None = ...,
        eap_exclude_peergrp: str | None = ...,
        eap_cert_auth: Literal["enable", "disable"] | None = ...,
        acct_verify: Literal["enable", "disable"] | None = ...,
        ppk: Literal["disable", "allow", "require"] | None = ...,
        ppk_secret: str | None = ...,
        ppk_identity: str | None = ...,
        wizard_type: Literal["custom", "dialup-forticlient", "dialup-ios", "dialup-android", "dialup-windows", "dialup-cisco", "static-fortigate", "dialup-fortigate", "static-cisco", "dialup-cisco-fw", "simplified-static-fortigate", "hub-fortigate-auto-discovery", "spoke-fortigate-auto-discovery", "fabric-overlay-orchestrator"] | None = ...,
        xauthtype: Literal["disable", "client", "pap", "chap", "auto"] | None = ...,
        reauth: Literal["disable", "enable"] | None = ...,
        authusr: str | None = ...,
        authpasswd: str | None = ...,
        group_authentication: Literal["enable", "disable"] | None = ...,
        group_authentication_secret: str | None = ...,
        authusrgrp: str | None = ...,
        mesh_selector_type: Literal["disable", "subnet", "host"] | None = ...,
        idle_timeout: Literal["enable", "disable"] | None = ...,
        shared_idle_timeout: Literal["enable", "disable"] | None = ...,
        idle_timeoutinterval: int | None = ...,
        ha_sync_esp_seqno: Literal["enable", "disable"] | None = ...,
        fgsp_sync: Literal["enable", "disable"] | None = ...,
        inbound_dscp_copy: Literal["enable", "disable"] | None = ...,
        nattraversal: Literal["enable", "disable", "forced"] | None = ...,
        esn: Literal["require", "allow", "disable"] | None = ...,
        fragmentation_mtu: int | None = ...,
        childless_ike: Literal["enable", "disable"] | None = ...,
        azure_ad_autoconnect: Literal["enable", "disable"] | None = ...,
        client_resume: Literal["enable", "disable"] | None = ...,
        client_resume_interval: int | None = ...,
        rekey: Literal["enable", "disable"] | None = ...,
        digital_signature_auth: Literal["enable", "disable"] | None = ...,
        signature_hash_alg: str | list[str] | None = ...,
        rsa_signature_format: Literal["pkcs1", "pss"] | None = ...,
        rsa_signature_hash_override: Literal["enable", "disable"] | None = ...,
        enforce_unique_id: Literal["disable", "keep-new", "keep-old"] | None = ...,
        cert_id_validation: Literal["enable", "disable"] | None = ...,
        fec_egress: Literal["enable", "disable"] | None = ...,
        fec_send_timeout: int | None = ...,
        fec_base: int | None = ...,
        fec_codec: Literal["rs", "xor"] | None = ...,
        fec_redundant: int | None = ...,
        fec_ingress: Literal["enable", "disable"] | None = ...,
        fec_receive_timeout: int | None = ...,
        fec_health_check: str | None = ...,
        fec_mapping_profile: str | None = ...,
        network_overlay: Literal["disable", "enable"] | None = ...,
        network_id: int | None = ...,
        dev_id_notification: Literal["disable", "enable"] | None = ...,
        dev_id: str | None = ...,
        loopback_asymroute: Literal["enable", "disable"] | None = ...,
        link_cost: int | None = ...,
        kms: str | None = ...,
        exchange_fgt_device_id: Literal["enable", "disable"] | None = ...,
        ipv6_auto_linklocal: Literal["enable", "disable"] | None = ...,
        ems_sn_check: Literal["enable", "disable"] | None = ...,
        cert_trust_store: Literal["local", "ems"] | None = ...,
        qkd: Literal["disable", "allow", "require"] | None = ...,
        qkd_hybrid: Literal["disable", "allow", "require"] | None = ...,
        qkd_profile: str | None = ...,
        transport: Literal["udp", "auto", "tcp"] | None = ...,
        fortinet_esp: Literal["enable", "disable"] | None = ...,
        auto_transport_threshold: int | None = ...,
        remote_gw_match: Literal["any", "ipmask", "iprange", "geography", "ztna"] | None = ...,
        remote_gw_subnet: str | None = ...,
        remote_gw_start_ip: str | None = ...,
        remote_gw_end_ip: str | None = ...,
        remote_gw_country: str | None = ...,
        remote_gw_ztna_tags: str | list[str] | list[dict[str, Any]] | list[Phase1RemotegwztnatagsItem] | None = ...,
        remote_gw6_match: Literal["any", "ipprefix", "iprange", "geography"] | None = ...,
        remote_gw6_subnet: str | None = ...,
        remote_gw6_start_ip: str | None = ...,
        remote_gw6_end_ip: str | None = ...,
        remote_gw6_country: str | None = ...,
        cert_peer_username_validation: Literal["none", "othername", "rfc822name", "cn"] | None = ...,
        cert_peer_username_strip: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Phase1Object: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: Phase1Payload | None = ...,
        name: str | None = ...,
        type: Literal["static", "dynamic", "ddns"] | None = ...,
        interface: str | None = ...,
        ike_version: Literal["1", "2"] | None = ...,
        remote_gw: str | None = ...,
        local_gw: str | None = ...,
        remotegw_ddns: str | None = ...,
        keylife: int | None = ...,
        certificate: str | list[str] | list[dict[str, Any]] | list[Phase1CertificateItem] | None = ...,
        authmethod: Literal["psk", "signature"] | None = ...,
        authmethod_remote: Literal["psk", "signature"] | None = ...,
        mode: Literal["aggressive", "main"] | None = ...,
        peertype: Literal["any", "one", "dialup", "peer", "peergrp"] | None = ...,
        peerid: str | None = ...,
        usrgrp: str | None = ...,
        peer: str | None = ...,
        peergrp: str | None = ...,
        mode_cfg: Literal["disable", "enable"] | None = ...,
        mode_cfg_allow_client_selector: Literal["disable", "enable"] | None = ...,
        assign_ip: Literal["disable", "enable"] | None = ...,
        assign_ip_from: Literal["range", "usrgrp", "dhcp", "name"] | None = ...,
        ipv4_start_ip: str | None = ...,
        ipv4_end_ip: str | None = ...,
        ipv4_netmask: str | None = ...,
        dhcp_ra_giaddr: str | None = ...,
        dhcp6_ra_linkaddr: str | None = ...,
        dns_mode: Literal["manual", "auto"] | None = ...,
        ipv4_dns_server1: str | None = ...,
        ipv4_dns_server2: str | None = ...,
        ipv4_dns_server3: str | None = ...,
        internal_domain_list: str | list[str] | list[dict[str, Any]] | list[Phase1InternaldomainlistItem] | None = ...,
        dns_suffix_search: str | list[str] | list[dict[str, Any]] | list[Phase1DnssuffixsearchItem] | None = ...,
        ipv4_wins_server1: str | None = ...,
        ipv4_wins_server2: str | None = ...,
        ipv4_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv4excluderangeItem] | None = ...,
        ipv4_split_include: str | None = ...,
        split_include_service: str | None = ...,
        ipv4_name: str | None = ...,
        ipv6_start_ip: str | None = ...,
        ipv6_end_ip: str | None = ...,
        ipv6_prefix: int | None = ...,
        ipv6_dns_server1: str | None = ...,
        ipv6_dns_server2: str | None = ...,
        ipv6_dns_server3: str | None = ...,
        ipv6_exclude_range: str | list[str] | list[dict[str, Any]] | list[Phase1Ipv6excluderangeItem] | None = ...,
        ipv6_split_include: str | None = ...,
        ipv6_name: str | None = ...,
        ip_delay_interval: int | None = ...,
        unity_support: Literal["disable", "enable"] | None = ...,
        domain: str | None = ...,
        banner: str | None = ...,
        include_local_lan: Literal["disable", "enable"] | None = ...,
        ipv4_split_exclude: str | None = ...,
        ipv6_split_exclude: str | None = ...,
        save_password: Literal["disable", "enable"] | None = ...,
        client_auto_negotiate: Literal["disable", "enable"] | None = ...,
        client_keep_alive: Literal["disable", "enable"] | None = ...,
        backup_gateway: str | list[str] | list[dict[str, Any]] | list[Phase1BackupgatewayItem] | None = ...,
        proposal: Literal["des-md5", "des-sha1", "des-sha256", "des-sha384", "des-sha512", "3des-md5", "3des-sha1", "3des-sha256", "3des-sha384", "3des-sha512", "aes128-md5", "aes128-sha1", "aes128-sha256", "aes128-sha384", "aes128-sha512", "aes128gcm-prfsha1", "aes128gcm-prfsha256", "aes128gcm-prfsha384", "aes128gcm-prfsha512", "aes192-md5", "aes192-sha1", "aes192-sha256", "aes192-sha384", "aes192-sha512", "aes256-md5", "aes256-sha1", "aes256-sha256", "aes256-sha384", "aes256-sha512", "aes256gcm-prfsha1", "aes256gcm-prfsha256", "aes256gcm-prfsha384", "aes256gcm-prfsha512", "chacha20poly1305-prfsha1", "chacha20poly1305-prfsha256", "chacha20poly1305-prfsha384", "chacha20poly1305-prfsha512", "aria128-md5", "aria128-sha1", "aria128-sha256", "aria128-sha384", "aria128-sha512", "aria192-md5", "aria192-sha1", "aria192-sha256", "aria192-sha384", "aria192-sha512", "aria256-md5", "aria256-sha1", "aria256-sha256", "aria256-sha384", "aria256-sha512", "seed-md5", "seed-sha1", "seed-sha256", "seed-sha384", "seed-sha512"] | list[str] | None = ...,
        add_route: Literal["disable", "enable"] | None = ...,
        add_gw_route: Literal["enable", "disable"] | None = ...,
        psksecret: str | None = ...,
        psksecret_remote: str | None = ...,
        keepalive: int | None = ...,
        distance: int | None = ...,
        priority: int | None = ...,
        localid: str | None = ...,
        localid_type: Literal["auto", "fqdn", "user-fqdn", "keyid", "address", "asn1dn"] | None = ...,
        auto_negotiate: Literal["enable", "disable"] | None = ...,
        negotiate_timeout: int | None = ...,
        fragmentation: Literal["enable", "disable"] | None = ...,
        dpd: Literal["disable", "on-idle", "on-demand"] | None = ...,
        dpd_retrycount: int | None = ...,
        dpd_retryinterval: str | None = ...,
        comments: str | None = ...,
        npu_offload: Literal["enable", "disable"] | None = ...,
        send_cert_chain: Literal["enable", "disable"] | None = ...,
        dhgrp: Literal["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31", "32"] | list[str] | None = ...,
        addke1: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke2: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke3: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke4: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke5: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke6: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        addke7: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"] | list[str] | None = ...,
        suite_b: Literal["disable", "suite-b-gcm-128", "suite-b-gcm-256"] | None = ...,
        eap: Literal["enable", "disable"] | None = ...,
        eap_identity: Literal["use-id-payload", "send-request"] | None = ...,
        eap_exclude_peergrp: str | None = ...,
        eap_cert_auth: Literal["enable", "disable"] | None = ...,
        acct_verify: Literal["enable", "disable"] | None = ...,
        ppk: Literal["disable", "allow", "require"] | None = ...,
        ppk_secret: str | None = ...,
        ppk_identity: str | None = ...,
        wizard_type: Literal["custom", "dialup-forticlient", "dialup-ios", "dialup-android", "dialup-windows", "dialup-cisco", "static-fortigate", "dialup-fortigate", "static-cisco", "dialup-cisco-fw", "simplified-static-fortigate", "hub-fortigate-auto-discovery", "spoke-fortigate-auto-discovery", "fabric-overlay-orchestrator"] | None = ...,
        xauthtype: Literal["disable", "client", "pap", "chap", "auto"] | None = ...,
        reauth: Literal["disable", "enable"] | None = ...,
        authusr: str | None = ...,
        authpasswd: str | None = ...,
        group_authentication: Literal["enable", "disable"] | None = ...,
        group_authentication_secret: str | None = ...,
        authusrgrp: str | None = ...,
        mesh_selector_type: Literal["disable", "subnet", "host"] | None = ...,
        idle_timeout: Literal["enable", "disable"] | None = ...,
        shared_idle_timeout: Literal["enable", "disable"] | None = ...,
        idle_timeoutinterval: int | None = ...,
        ha_sync_esp_seqno: Literal["enable", "disable"] | None = ...,
        fgsp_sync: Literal["enable", "disable"] | None = ...,
        inbound_dscp_copy: Literal["enable", "disable"] | None = ...,
        nattraversal: Literal["enable", "disable", "forced"] | None = ...,
        esn: Literal["require", "allow", "disable"] | None = ...,
        fragmentation_mtu: int | None = ...,
        childless_ike: Literal["enable", "disable"] | None = ...,
        azure_ad_autoconnect: Literal["enable", "disable"] | None = ...,
        client_resume: Literal["enable", "disable"] | None = ...,
        client_resume_interval: int | None = ...,
        rekey: Literal["enable", "disable"] | None = ...,
        digital_signature_auth: Literal["enable", "disable"] | None = ...,
        signature_hash_alg: Literal["sha1", "sha2-256", "sha2-384", "sha2-512"] | list[str] | None = ...,
        rsa_signature_format: Literal["pkcs1", "pss"] | None = ...,
        rsa_signature_hash_override: Literal["enable", "disable"] | None = ...,
        enforce_unique_id: Literal["disable", "keep-new", "keep-old"] | None = ...,
        cert_id_validation: Literal["enable", "disable"] | None = ...,
        fec_egress: Literal["enable", "disable"] | None = ...,
        fec_send_timeout: int | None = ...,
        fec_base: int | None = ...,
        fec_codec: Literal["rs", "xor"] | None = ...,
        fec_redundant: int | None = ...,
        fec_ingress: Literal["enable", "disable"] | None = ...,
        fec_receive_timeout: int | None = ...,
        fec_health_check: str | None = ...,
        fec_mapping_profile: str | None = ...,
        network_overlay: Literal["disable", "enable"] | None = ...,
        network_id: int | None = ...,
        dev_id_notification: Literal["disable", "enable"] | None = ...,
        dev_id: str | None = ...,
        loopback_asymroute: Literal["enable", "disable"] | None = ...,
        link_cost: int | None = ...,
        kms: str | None = ...,
        exchange_fgt_device_id: Literal["enable", "disable"] | None = ...,
        ipv6_auto_linklocal: Literal["enable", "disable"] | None = ...,
        ems_sn_check: Literal["enable", "disable"] | None = ...,
        cert_trust_store: Literal["local", "ems"] | None = ...,
        qkd: Literal["disable", "allow", "require"] | None = ...,
        qkd_hybrid: Literal["disable", "allow", "require"] | None = ...,
        qkd_profile: str | None = ...,
        transport: Literal["udp", "auto", "tcp"] | None = ...,
        fortinet_esp: Literal["enable", "disable"] | None = ...,
        auto_transport_threshold: int | None = ...,
        remote_gw_match: Literal["any", "ipmask", "iprange", "geography", "ztna"] | None = ...,
        remote_gw_subnet: str | None = ...,
        remote_gw_start_ip: str | None = ...,
        remote_gw_end_ip: str | None = ...,
        remote_gw_country: str | None = ...,
        remote_gw_ztna_tags: str | list[str] | list[dict[str, Any]] | list[Phase1RemotegwztnatagsItem] | None = ...,
        remote_gw6_match: Literal["any", "ipprefix", "iprange", "geography"] | None = ...,
        remote_gw6_subnet: str | None = ...,
        remote_gw6_start_ip: str | None = ...,
        remote_gw6_end_ip: str | None = ...,
        remote_gw6_country: str | None = ...,
        cert_peer_username_validation: Literal["none", "othername", "rfc822name", "cn"] | None = ...,
        cert_peer_username_strip: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Phase1",
    "Phase1Payload",
    "Phase1Response",
    "Phase1Object",
]


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: router/ospf
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class OspfAreaItem:
    """Nested item for area field - supports attribute access."""
    id: str
    shortcut: Literal["disable", "enable", "default"]
    authentication: Literal["none", "text", "message-digest"]
    default_cost: int
    nssa_translator_role: Literal["candidate", "never", "always"]
    stub_type: Literal["no-summary", "summary"]
    type: Literal["regular", "nssa", "stub"]
    nssa_default_information_originate: Literal["enable", "always", "disable"]
    nssa_default_information_originate_metric: int
    nssa_default_information_originate_metric_type: Literal["1", "2"]
    nssa_redistribution: Literal["enable", "disable"]
    comments: str
    range: str
    virtual_link: str
    filter_list: str


class OspfOspfinterfaceItem:
    """Nested item for ospf-interface field - supports attribute access."""
    name: str
    comments: str
    interface: str
    ip: str
    linkdown_fast_failover: Literal["enable", "disable"]
    authentication: Literal["none", "text", "message-digest"]
    authentication_key: str
    keychain: str
    prefix_length: int
    retransmit_interval: int
    transmit_delay: int
    cost: int
    priority: int
    dead_interval: int
    hello_interval: int
    hello_multiplier: int
    database_filter_out: Literal["enable", "disable"]
    mtu: int
    mtu_ignore: Literal["enable", "disable"]
    network_type: Literal["broadcast", "non-broadcast", "point-to-point", "point-to-multipoint", "point-to-multipoint-non-broadcast"]
    bfd: Literal["global", "enable", "disable"]
    status: Literal["disable", "enable"]
    resync_timeout: int
    md5_keys: str


class OspfNetworkItem:
    """Nested item for network field - supports attribute access."""
    id: int
    prefix: str
    area: str
    comments: str


class OspfNeighborItem:
    """Nested item for neighbor field - supports attribute access."""
    id: int
    ip: str
    poll_interval: int
    cost: int
    priority: int


class OspfPassiveinterfaceItem:
    """Nested item for passive-interface field - supports attribute access."""
    name: str


class OspfSummaryaddressItem:
    """Nested item for summary-address field - supports attribute access."""
    id: int
    prefix: str
    tag: int
    advertise: Literal["disable", "enable"]


class OspfDistributelistItem:
    """Nested item for distribute-list field - supports attribute access."""
    id: int
    access_list: str
    protocol: Literal["connected", "static", "rip"]


class OspfRedistributeItem:
    """Nested item for redistribute field - supports attribute access."""
    name: str
    status: Literal["enable", "disable"]
    metric: int
    routemap: str
    metric_type: Literal["1", "2"]
    tag: int


class OspfPayload(TypedDict, total=False):
    """Payload type for Ospf operations."""
    abr_type: Literal["cisco", "ibm", "shortcut", "standard"]
    auto_cost_ref_bandwidth: int
    distance_external: int
    distance_inter_area: int
    distance_intra_area: int
    database_overflow: Literal["enable", "disable"]
    database_overflow_max_lsas: int
    database_overflow_time_to_recover: int
    default_information_originate: Literal["enable", "always", "disable"]
    default_information_metric: int
    default_information_metric_type: Literal["1", "2"]
    default_information_route_map: str
    default_metric: int
    distance: int
    lsa_refresh_interval: int
    rfc1583_compatible: Literal["enable", "disable"]
    router_id: str
    spf_timers: str
    bfd: Literal["enable", "disable"]
    log_neighbour_changes: Literal["enable", "disable"]
    distribute_list_in: str
    distribute_route_map_in: str
    restart_mode: Literal["none", "lls", "graceful-restart"]
    restart_period: int
    restart_on_topology_change: Literal["enable", "disable"]
    area: str | list[str] | list[dict[str, Any]] | list[OspfAreaItem]
    ospf_interface: str | list[str] | list[dict[str, Any]] | list[OspfOspfinterfaceItem]
    network: str | list[str] | list[dict[str, Any]] | list[OspfNetworkItem]
    neighbor: str | list[str] | list[dict[str, Any]] | list[OspfNeighborItem]
    passive_interface: str | list[str] | list[dict[str, Any]] | list[OspfPassiveinterfaceItem]
    summary_address: str | list[str] | list[dict[str, Any]] | list[OspfSummaryaddressItem]
    distribute_list: str | list[str] | list[dict[str, Any]] | list[OspfDistributelistItem]
    redistribute: str | list[str] | list[dict[str, Any]] | list[OspfRedistributeItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class OspfResponse(TypedDict, total=False):
    """Response type for Ospf - use with .dict property for typed dict access."""
    abr_type: Literal["cisco", "ibm", "shortcut", "standard"]
    auto_cost_ref_bandwidth: int
    distance_external: int
    distance_inter_area: int
    distance_intra_area: int
    database_overflow: Literal["enable", "disable"]
    database_overflow_max_lsas: int
    database_overflow_time_to_recover: int
    default_information_originate: Literal["enable", "always", "disable"]
    default_information_metric: int
    default_information_metric_type: Literal["1", "2"]
    default_information_route_map: str
    default_metric: int
    distance: int
    lsa_refresh_interval: int
    rfc1583_compatible: Literal["enable", "disable"]
    router_id: str
    spf_timers: str
    bfd: Literal["enable", "disable"]
    log_neighbour_changes: Literal["enable", "disable"]
    distribute_list_in: str
    distribute_route_map_in: str
    restart_mode: Literal["none", "lls", "graceful-restart"]
    restart_period: int
    restart_on_topology_change: Literal["enable", "disable"]
    area: list[OspfAreaItem]
    ospf_interface: list[OspfOspfinterfaceItem]
    network: list[OspfNetworkItem]
    neighbor: list[OspfNeighborItem]
    passive_interface: list[OspfPassiveinterfaceItem]
    summary_address: list[OspfSummaryaddressItem]
    distribute_list: list[OspfDistributelistItem]
    redistribute: list[OspfRedistributeItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class OspfObject(FortiObject):
    """Typed FortiObject for Ospf with field access."""
    abr_type: Literal["cisco", "ibm", "shortcut", "standard"]
    auto_cost_ref_bandwidth: int
    distance_external: int
    distance_inter_area: int
    distance_intra_area: int
    database_overflow: Literal["enable", "disable"]
    database_overflow_max_lsas: int
    database_overflow_time_to_recover: int
    default_information_originate: Literal["enable", "always", "disable"]
    default_information_metric: int
    default_information_metric_type: Literal["1", "2"]
    default_information_route_map: str
    default_metric: int
    distance: int
    lsa_refresh_interval: int
    rfc1583_compatible: Literal["enable", "disable"]
    router_id: str
    spf_timers: str
    bfd: Literal["enable", "disable"]
    log_neighbour_changes: Literal["enable", "disable"]
    distribute_list_in: str
    distribute_route_map_in: str
    restart_mode: Literal["none", "lls", "graceful-restart"]
    restart_period: int
    restart_on_topology_change: Literal["enable", "disable"]
    area: list[OspfAreaItem]
    ospf_interface: list[OspfOspfinterfaceItem]
    network: list[OspfNetworkItem]
    neighbor: list[OspfNeighborItem]
    passive_interface: list[OspfPassiveinterfaceItem]
    summary_address: list[OspfSummaryaddressItem]
    distribute_list: list[OspfDistributelistItem]
    redistribute: list[OspfRedistributeItem]


# ================================================================
# Main Endpoint Class
# ================================================================

class Ospf:
    """
    
    Endpoint: router/ospf
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> OspfObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: OspfPayload | None = ...,
        abr_type: Literal["cisco", "ibm", "shortcut", "standard"] | None = ...,
        auto_cost_ref_bandwidth: int | None = ...,
        distance_external: int | None = ...,
        distance_inter_area: int | None = ...,
        distance_intra_area: int | None = ...,
        database_overflow: Literal["enable", "disable"] | None = ...,
        database_overflow_max_lsas: int | None = ...,
        database_overflow_time_to_recover: int | None = ...,
        default_information_originate: Literal["enable", "always", "disable"] | None = ...,
        default_information_metric: int | None = ...,
        default_information_metric_type: Literal["1", "2"] | None = ...,
        default_information_route_map: str | None = ...,
        default_metric: int | None = ...,
        distance: int | None = ...,
        lsa_refresh_interval: int | None = ...,
        rfc1583_compatible: Literal["enable", "disable"] | None = ...,
        router_id: str | None = ...,
        spf_timers: str | None = ...,
        bfd: Literal["enable", "disable"] | None = ...,
        log_neighbour_changes: Literal["enable", "disable"] | None = ...,
        distribute_list_in: str | None = ...,
        distribute_route_map_in: str | None = ...,
        restart_mode: Literal["none", "lls", "graceful-restart"] | None = ...,
        restart_period: int | None = ...,
        restart_on_topology_change: Literal["enable", "disable"] | None = ...,
        area: str | list[str] | list[dict[str, Any]] | list[OspfAreaItem] | None = ...,
        ospf_interface: str | list[str] | list[dict[str, Any]] | list[OspfOspfinterfaceItem] | None = ...,
        network: str | list[str] | list[dict[str, Any]] | list[OspfNetworkItem] | None = ...,
        neighbor: str | list[str] | list[dict[str, Any]] | list[OspfNeighborItem] | None = ...,
        passive_interface: str | list[str] | list[dict[str, Any]] | list[OspfPassiveinterfaceItem] | None = ...,
        summary_address: str | list[str] | list[dict[str, Any]] | list[OspfSummaryaddressItem] | None = ...,
        distribute_list: str | list[str] | list[dict[str, Any]] | list[OspfDistributelistItem] | None = ...,
        redistribute: str | list[str] | list[dict[str, Any]] | list[OspfRedistributeItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> OspfObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: OspfPayload | None = ...,
        abr_type: Literal["cisco", "ibm", "shortcut", "standard"] | None = ...,
        auto_cost_ref_bandwidth: int | None = ...,
        distance_external: int | None = ...,
        distance_inter_area: int | None = ...,
        distance_intra_area: int | None = ...,
        database_overflow: Literal["enable", "disable"] | None = ...,
        database_overflow_max_lsas: int | None = ...,
        database_overflow_time_to_recover: int | None = ...,
        default_information_originate: Literal["enable", "always", "disable"] | None = ...,
        default_information_metric: int | None = ...,
        default_information_metric_type: Literal["1", "2"] | None = ...,
        default_information_route_map: str | None = ...,
        default_metric: int | None = ...,
        distance: int | None = ...,
        lsa_refresh_interval: int | None = ...,
        rfc1583_compatible: Literal["enable", "disable"] | None = ...,
        router_id: str | None = ...,
        spf_timers: str | None = ...,
        bfd: Literal["enable", "disable"] | None = ...,
        log_neighbour_changes: Literal["enable", "disable"] | None = ...,
        distribute_list_in: str | None = ...,
        distribute_route_map_in: str | None = ...,
        restart_mode: Literal["none", "lls", "graceful-restart"] | None = ...,
        restart_period: int | None = ...,
        restart_on_topology_change: Literal["enable", "disable"] | None = ...,
        area: str | list[str] | list[dict[str, Any]] | list[OspfAreaItem] | None = ...,
        ospf_interface: str | list[str] | list[dict[str, Any]] | list[OspfOspfinterfaceItem] | None = ...,
        network: str | list[str] | list[dict[str, Any]] | list[OspfNetworkItem] | None = ...,
        neighbor: str | list[str] | list[dict[str, Any]] | list[OspfNeighborItem] | None = ...,
        passive_interface: str | list[str] | list[dict[str, Any]] | list[OspfPassiveinterfaceItem] | None = ...,
        summary_address: str | list[str] | list[dict[str, Any]] | list[OspfSummaryaddressItem] | None = ...,
        distribute_list: str | list[str] | list[dict[str, Any]] | list[OspfDistributelistItem] | None = ...,
        redistribute: str | list[str] | list[dict[str, Any]] | list[OspfRedistributeItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Ospf",
    "OspfPayload",
    "OspfResponse",
    "OspfObject",
]


"""Type stubs for INFO category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .query import Query
    from .thumbnail import Thumbnail
    from .thumbnail_file import ThumbnailFile

__all__ = [
    "Query",
    "Thumbnail",
    "ThumbnailFile",
    "Info",
]


class Info:
    """INFO API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    query: Query
    thumbnail: Thumbnail
    thumbnail_file: ThumbnailFile

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize info category with HTTP client."""
        ...



"""
Pydantic Models for CMDB - switch_controller/snmp_user

Runtime validation models for switch_controller/snmp_user configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SnmpUserAuthProtoEnum(str, Enum):
    """Allowed values for auth_proto field."""
    MD5 = "md5"
    SHA1 = "sha1"
    SHA224 = "sha224"
    SHA256 = "sha256"
    SHA384 = "sha384"
    SHA512 = "sha512"

class SnmpUserPrivProtoEnum(str, Enum):
    """Allowed values for priv_proto field."""
    AES128 = "aes128"
    AES192 = "aes192"
    AES192C = "aes192c"
    AES256 = "aes256"
    AES256C = "aes256c"
    DES = "des"


# ============================================================================
# Main Model
# ============================================================================

class SnmpUserModel(BaseModel):
    """
    Pydantic model for switch_controller/snmp_user configuration.
    
    Configure FortiSwitch SNMP v3 users globally.
    
    Validation Rules:        - name: max_length=32 pattern=        - queries: pattern=        - query_port: min=0 max=65535 pattern=        - security_level: pattern=        - auth_proto: pattern=        - auth_pwd: max_length=128 pattern=        - priv_proto: pattern=        - priv_pwd: max_length=128 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=32, default=None, description="SNMP user name.")    
    queries: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable SNMP queries for this user.")    
    query_port: int | None = Field(ge=0, le=65535, default=161, description="SNMPv3 query port (default = 161).")    
    security_level: Literal["no-auth-no-priv", "auth-no-priv", "auth-priv"] | None = Field(default="no-auth-no-priv", description="Security level for message authentication and encryption.")    
    auth_proto: SnmpUserAuthProtoEnum | None = Field(default=SnmpUserAuthProtoEnum.SHA256, description="Authentication protocol.")    
    auth_pwd: Any = Field(max_length=128, description="Password for authentication protocol.")    
    priv_proto: SnmpUserPrivProtoEnum | None = Field(default=SnmpUserPrivProtoEnum.AES128, description="Privacy (encryption) protocol.")    
    priv_pwd: Any = Field(max_length=128, description="Password for privacy (encryption) protocol.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SnmpUserModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SnmpUserModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.883959Z
# ============================================================================


"""
Interactive help system for FortiOS API endpoints.

Provides context-aware help based on endpoint category (cmdb, monitor, log, service).
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from typing import Protocol

    class EndpointProtocol(Protocol):
        """Protocol for endpoint objects that support help."""

        def schema(self) -> dict[str, Any]: ...
        def fields(self) -> list[str]: ...
        def required_fields(self) -> list[str]: ...
        def defaults(self) -> dict[str, Any]: ...


def help(endpoint: Any, show_fields: bool = False) -> None:
    """
    Display comprehensive help for any FortiOS API endpoint.

    Provides context-aware documentation based on endpoint category:
    - CMDB: Configuration database operations
    - Monitor: Runtime status and statistics
    - Log: Log queries and retrieval
    - Service: Service operations and actions

    Args:
        endpoint: The API endpoint object (e.g., fgt.api.cmdb.firewall.policy)
        show_fields: Whether to list all available fields (default: False)

    Examples:
        >>> from hfortix_fortios import FortiOS, help
        >>> fgt = FortiOS('192.168.1.1', token='xxx')
        >>>
        >>> # Get help for firewall policy endpoint
        >>> help(fgt.api.cmdb.firewall.policy)
        >>>
        >>> # Show help with field listing
        >>> help(fgt.api.cmdb.system.interface, show_fields=True)
    """
    print("\n" + "=" * 80)
    print("FORTIOS API ENDPOINT HELP")
    print("=" * 80)

    # Get the class to call classmethod schema()
    endpoint_class = (
        endpoint if isinstance(endpoint, type) else endpoint.__class__
    )

    # Get schema to determine endpoint type
    schema = endpoint_class.schema()
    category = schema.get("category", "unknown")
    endpoint_path = schema.get("endpoint", "unknown")
    help_text = schema.get("help", "No description available")
    mkey = schema.get("mkey")
    mkey_type = schema.get("mkey_type")

    # Basic info
    print(f"\n📍 ENDPOINT: {endpoint_path}")
    print(f"📂 CATEGORY: {category.upper()}")
    print(f"ℹ️  DESCRIPTION: {help_text}")
    if mkey:
        print(f"🔑 PRIMARY KEY: {mkey} ({mkey_type})")

    # Field summary
    total_fields = schema.get("total_fields", 0)
    required_count = schema.get("required_fields_count", 0)
    defaults_count = schema.get("fields_with_defaults_count", 0)

    if total_fields:
        print("\n📊 FIELDS:")
        print(f"   Total: {total_fields}")
        print(f"   Required: {required_count}")
        print(f"   With Defaults: {defaults_count}")

    # Get available methods dynamically
    import inspect

    methods = []
    for name in dir(endpoint):
        if name.startswith("_"):
            continue
        attr = getattr(endpoint, name, None)
        if callable(attr) or (
            inspect.ismethod(attr) if hasattr(inspect, "ismethod") else False
        ):
            methods.append(name)

    # Categorize methods
    capabilities = [m for m in methods if m.startswith("SUPPORTS_")]

    # Known operation methods
    known_operations = [
        "get",
        "post",
        "put",
        "delete",
        "patch",
        "clone",
        "move",
        "set",
        "create",
        "update",
    ]
    operations = [m for m in methods if m in known_operations]

    # Known info/metadata methods
    known_info = [
        "help",
        "schema",
        "get_schema",
        "fields",
        "field_info",
        "required_fields",
        "defaults",
        "exists",
        "validate_field",
    ]
    info_methods = [m for m in methods if m in known_info]

    # Show supported capabilities
    print("\n📋 CAPABILITIES:")
    supported = [cap for cap in capabilities if getattr(endpoint, cap, False)]
    cap_names = [cap.replace("SUPPORTS_", "").lower() for cap in supported]
    if cap_names:
        print(f"   {', '.join(cap_names)}")
    else:
        print("   None")

    # Category-specific operation descriptions
    op_descriptions = _get_operation_descriptions(category)

    print("\n⚙️  OPERATIONS:")
    if operations:
        for op in operations:
            desc = op_descriptions.get(op, "Perform operation")
            print(f"   • {op}() - {desc}")
    else:
        print("   None")

    # Show info methods
    info_desc = {
        "help": "Show this help information",
        "schema": "Get full schema with field definitions",
        "get_schema": "Alias for schema()",
        "fields": "List all available field names",
        "field_info": "Get details about specific field",
        "required_fields": "List required fields for creation",
        "defaults": "Get default values for fields",
        "exists": "Check if object exists by ID",
        "validate_field": "Validate field value before sending",
    }

    print("\n📖 INFO METHODS:")
    if info_methods:
        for info in info_methods:
            desc = info_desc.get(info, "Information method")
            print(f"   • {info}() - {desc}")
    else:
        print("   None")

    print("\n" + "=" * 80)


def _get_operation_descriptions(category: str) -> dict[str, str]:
    """
    Get category-specific operation descriptions.

    Args:
        category: Endpoint category (cmdb, monitor, log, service)

    Returns:
        Dictionary mapping operation names to descriptions
    """
    if category == "cmdb":
        return {
            "get": "Retrieve configuration objects",
            "post": "Create new configuration object",
            "create": "Create new configuration object",
            "put": "Update existing configuration object",
            "update": "Update existing configuration object",
            "patch": "Partial update of configuration object",
            "delete": "Delete configuration object",
            "set": "Create or update object (upsert)",
            "clone": "Duplicate existing object",
            "move": "Reorder object position",
        }
    elif category == "monitor":
        return {
            "get": "Retrieve status/statistics",
            "post": "Perform action or query",
            "put": "Update runtime settings",
            "patch": "Partial update of runtime settings",
            "delete": "Clear/reset data",
        }
    elif category == "log":
        return {
            "get": "Retrieve log entries",
            "post": "Query logs with filters",
        }
    elif category == "service":
        return {
            "get": "Query service data",
            "post": "Trigger service action or query with parameters",
            "put": "Execute service operation",
            "patch": "Partial service operation",
            "delete": "Remove service data",
        }
    else:
        return {
            "get": "Retrieve data",
            "post": "Create/update/perform action",
            "create": "Create new resource",
            "put": "Update data",
            "update": "Update existing resource",
            "patch": "Partial update",
            "delete": "Delete data",
            "clone": "Duplicate resource",
            "move": "Reorder or move resource",
        }



"""
Pydantic Models for CMDB - firewall/proxy_address

Runtime validation models for firewall/proxy_address configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum
from uuid import UUID

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class ProxyAddressTaggingTags(BaseModel):
    """
    Child table model for tagging.tags.
    
    Tags.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="Tag name.")  # datasource: ['system.object-tagging.tags.name']
class ProxyAddressTagging(BaseModel):
    """
    Child table model for tagging.
    
    Config object tagging.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=63, default=None, description="Tagging entry name.")    
    category: str | None = Field(max_length=63, default=None, description="Tag category.")  # datasource: ['system.object-tagging.category']    
    tags: list[ProxyAddressTaggingTags] = Field(default_factory=list, description="Tags.")
class ProxyAddressHeaderGroup(BaseModel):
    """
    Child table model for header-group.
    
    HTTP header group.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    header_name: str = Field(max_length=79, description="HTTP header.")    
    header: str = Field(max_length=255, description="HTTP header regular expression.")    
    case_sensitivity: Literal["disable", "enable"] | None = Field(default="disable", description="Case sensitivity in pattern.")
class ProxyAddressCategory(BaseModel):
    """
    Child table model for category.
    
    FortiGuard category ID.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="FortiGuard category ID.")
class ProxyAddressApplication(BaseModel):
    """
    Child table model for application.
    
    SaaS application.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="SaaS application name.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class ProxyAddressTypeEnum(str, Enum):
    """Allowed values for type_ field."""
    HOST_REGEX = "host-regex"
    URL = "url"
    CATEGORY = "category"
    METHOD = "method"
    UA = "ua"
    HEADER = "header"
    SRC_ADVANCED = "src-advanced"
    DST_ADVANCED = "dst-advanced"
    SAAS = "saas"

class ProxyAddressMethodEnum(str, Enum):
    """Allowed values for method field."""
    GET = "get"
    POST = "post"
    PUT = "put"
    HEAD = "head"
    CONNECT = "connect"
    TRACE = "trace"
    OPTIONS = "options"
    DELETE = "delete"
    UPDATE = "update"
    PATCH = "patch"
    OTHER = "other"

class ProxyAddressUaEnum(str, Enum):
    """Allowed values for ua field."""
    CHROME = "chrome"
    MS = "ms"
    FIREFOX = "firefox"
    SAFARI = "safari"
    IE = "ie"
    EDGE = "edge"
    OTHER = "other"


# ============================================================================
# Main Model
# ============================================================================

class ProxyAddressModel(BaseModel):
    """
    Pydantic model for firewall/proxy_address configuration.
    
    Configure web proxy address.
    
    Validation Rules:        - name: max_length=79 pattern=        - uuid: pattern=        - type_: pattern=        - host: max_length=79 pattern=        - host_regex: max_length=255 pattern=        - path: max_length=255 pattern=        - query: max_length=255 pattern=        - referrer: pattern=        - category: pattern=        - method: pattern=        - ua: pattern=        - ua_min_ver: max_length=63 pattern=        - ua_max_ver: max_length=63 pattern=        - header_name: max_length=79 pattern=        - header: max_length=255 pattern=        - case_sensitivity: pattern=        - header_group: pattern=        - color: min=0 max=32 pattern=        - tagging: pattern=        - comment: max_length=255 pattern=        - application: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=79, default=None, description="Address name.")    
    uuid: str | None = Field(default="00000000-0000-0000-0000-000000000000", description="Universally Unique Identifier (UUID; automatically assigned but can be manually reset).")    
    type_: ProxyAddressTypeEnum | None = Field(default=ProxyAddressTypeEnum.URL, serialization_alias="type", description="Proxy address type.")    
    host: str = Field(max_length=79, description="Address object for the host.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name', 'firewall.proxy-address.name', 'firewall.vipgrp.name', 'firewall.vip.name']    
    host_regex: str | None = Field(max_length=255, default=None, description="Host name as a regular expression.")    
    path: str | None = Field(max_length=255, default=None, description="URL path as a regular expression.")    
    query: str | None = Field(max_length=255, default=None, description="Match the query part of the URL as a regular expression.")    
    referrer: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable use of referrer field in the HTTP header to match the address.")    
    category: list[ProxyAddressCategory] = Field(default_factory=list, description="FortiGuard category ID.")    
    method: list[ProxyAddressMethodEnum] = Field(default_factory=list, description="HTTP request methods to be used.")    
    ua: list[ProxyAddressUaEnum] = Field(default_factory=list, description="Names of browsers to be used as user agent.")    
    ua_min_ver: str | None = Field(max_length=63, default=None, description="Minimum version of the user agent specified in dotted notation. For example, use 90.0.1 with the ua field set to \"chrome\" to require Google Chrome's minimum version must be 90.0.1.")    
    ua_max_ver: str | None = Field(max_length=63, default=None, description="Maximum version of the user agent specified in dotted notation. For example, use 120 with the ua field set to \"chrome\" to require Google Chrome's maximum version must be 120.")    
    header_name: str | None = Field(max_length=79, default=None, description="Name of HTTP header.")    
    header: str | None = Field(max_length=255, default=None, description="HTTP header name as a regular expression.")    
    case_sensitivity: Literal["disable", "enable"] | None = Field(default="disable", description="Enable to make the pattern case sensitive.")    
    header_group: list[ProxyAddressHeaderGroup] = Field(default_factory=list, description="HTTP header group.")    
    color: int | None = Field(ge=0, le=32, default=0, description="Integer value to determine the color of the icon in the GUI (1 - 32, default = 0, which sets value to 1).")    
    tagging: list[ProxyAddressTagging] = Field(default_factory=list, description="Config object tagging.")    
    comment: str | None = Field(max_length=255, default=None, description="Optional comments.")    
    application: list[ProxyAddressApplication] = Field(description="SaaS application.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('host')
    @classmethod
    def validate_host(cls, v: Any) -> Any:
        """
        Validate host field.
        
        Datasource: ['firewall.address.name', 'firewall.addrgrp.name', 'firewall.proxy-address.name', 'firewall.vipgrp.name', 'firewall.vip.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ProxyAddressModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_host_references(self, client: Any) -> list[str]:
        """
        Validate host references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        - firewall/addrgrp        - firewall/proxy-address        - firewall/vipgrp        - firewall/vip        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProxyAddressModel(
            ...     host="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_host_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.proxy_address.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "host", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address.exists(value):
            found = True
        elif await client.api.cmdb.firewall.addrgrp.exists(value):
            found = True
        elif await client.api.cmdb.firewall.proxy_address.exists(value):
            found = True
        elif await client.api.cmdb.firewall.vipgrp.exists(value):
            found = True
        elif await client.api.cmdb.firewall.vip.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Host '{value}' not found in "
                "firewall/address or firewall/addrgrp or firewall/proxy-address or firewall/vipgrp or firewall/vip"
            )        
        return errors    
    async def validate_tagging_references(self, client: Any) -> list[str]:
        """
        Validate tagging references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/object-tagging        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProxyAddressModel(
            ...     tagging=[{"category": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_tagging_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.proxy_address.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "tagging", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("category")
            else:
                value = getattr(item, "category", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.object_tagging.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Tagging '{value}' not found in "
                    "system/object-tagging"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_host_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_tagging_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ProxyAddressModel",    "ProxyAddressCategory",    "ProxyAddressHeaderGroup",    "ProxyAddressTagging",    "ProxyAddressTagging.Tags",    "ProxyAddressApplication",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:10.923548Z
# ============================================================================


"""FortiOS CMDB - LteModem category"""

from .status import Status
from .upgrade import Upgrade
from .upload import Upload

__all__ = [
    "LteModem",
    "Status",
    "Upgrade",
    "Upload",
]


class LteModem:
    """LteModem endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """LteModem endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.status = Status(client)
        self.upgrade = Upgrade(client)
        self.upload = Upload(client)



"""
Pydantic Models for CMDB - system/ssh_config

Runtime validation models for system/ssh_config configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SshConfigSshKexAlgoEnum(str, Enum):
    """Allowed values for ssh_kex_algo field."""
    DIFFIE_HELLMAN_GROUP1_SHA1 = "diffie-hellman-group1-sha1"
    DIFFIE_HELLMAN_GROUP14_SHA1 = "diffie-hellman-group14-sha1"
    DIFFIE_HELLMAN_GROUP14_SHA256 = "diffie-hellman-group14-sha256"
    DIFFIE_HELLMAN_GROUP16_SHA512 = "diffie-hellman-group16-sha512"
    DIFFIE_HELLMAN_GROUP18_SHA512 = "diffie-hellman-group18-sha512"
    DIFFIE_HELLMAN_GROUP_EXCHANGE_SHA1 = "diffie-hellman-group-exchange-sha1"
    DIFFIE_HELLMAN_GROUP_EXCHANGE_SHA256 = "diffie-hellman-group-exchange-sha256"
    CURVE25519_SHA256LIBSSH_ORG = "curve25519-sha256@libssh.org"
    ECDH_SHA2_NISTP256 = "ecdh-sha2-nistp256"
    ECDH_SHA2_NISTP384 = "ecdh-sha2-nistp384"
    ECDH_SHA2_NISTP521 = "ecdh-sha2-nistp521"

class SshConfigSshEncAlgoEnum(str, Enum):
    """Allowed values for ssh_enc_algo field."""
    CHACHA20_POLY1305OPENSSH_COM = "chacha20-poly1305@openssh.com"
    AES128_CTR = "aes128-ctr"
    AES192_CTR = "aes192-ctr"
    AES256_CTR = "aes256-ctr"
    ARCFOUR256 = "arcfour256"
    ARCFOUR128 = "arcfour128"
    AES128_CBC = "aes128-cbc"
    V_3DES_CBC = "3des-cbc"
    BLOWFISH_CBC = "blowfish-cbc"
    CAST128_CBC = "cast128-cbc"
    AES192_CBC = "aes192-cbc"
    AES256_CBC = "aes256-cbc"
    ARCFOUR = "arcfour"
    RIJNDAEL_CBCLYSATOR_LIU_SE = "rijndael-cbc@lysator.liu.se"
    AES128_GCMOPENSSH_COM = "aes128-gcm@openssh.com"
    AES256_GCMOPENSSH_COM = "aes256-gcm@openssh.com"

class SshConfigSshMacAlgoEnum(str, Enum):
    """Allowed values for ssh_mac_algo field."""
    HMAC_MD5 = "hmac-md5"
    HMAC_MD5_ETMOPENSSH_COM = "hmac-md5-etm@openssh.com"
    HMAC_MD5_96 = "hmac-md5-96"
    HMAC_MD5_96_ETMOPENSSH_COM = "hmac-md5-96-etm@openssh.com"
    HMAC_SHA1 = "hmac-sha1"
    HMAC_SHA1_ETMOPENSSH_COM = "hmac-sha1-etm@openssh.com"
    HMAC_SHA2_256 = "hmac-sha2-256"
    HMAC_SHA2_256_ETMOPENSSH_COM = "hmac-sha2-256-etm@openssh.com"
    HMAC_SHA2_512 = "hmac-sha2-512"
    HMAC_SHA2_512_ETMOPENSSH_COM = "hmac-sha2-512-etm@openssh.com"
    HMAC_RIPEMD160 = "hmac-ripemd160"
    HMAC_RIPEMD160OPENSSH_COM = "hmac-ripemd160@openssh.com"
    HMAC_RIPEMD160_ETMOPENSSH_COM = "hmac-ripemd160-etm@openssh.com"
    UMAC_64OPENSSH_COM = "umac-64@openssh.com"
    UMAC_128OPENSSH_COM = "umac-128@openssh.com"
    UMAC_64_ETMOPENSSH_COM = "umac-64-etm@openssh.com"
    UMAC_128_ETMOPENSSH_COM = "umac-128-etm@openssh.com"

class SshConfigSshHskAlgoEnum(str, Enum):
    """Allowed values for ssh_hsk_algo field."""
    SSH_RSA = "ssh-rsa"
    ECDSA_SHA2_NISTP521 = "ecdsa-sha2-nistp521"
    ECDSA_SHA2_NISTP384 = "ecdsa-sha2-nistp384"
    ECDSA_SHA2_NISTP256 = "ecdsa-sha2-nistp256"
    RSA_SHA2_256 = "rsa-sha2-256"
    RSA_SHA2_512 = "rsa-sha2-512"
    SSH_ED25519 = "ssh-ed25519"


# ============================================================================
# Main Model
# ============================================================================

class SshConfigModel(BaseModel):
    """
    Pydantic model for system/ssh_config configuration.
    
    Configure SSH config.
    
    Validation Rules:        - ssh_kex_algo: pattern=        - ssh_enc_algo: pattern=        - ssh_mac_algo: pattern=        - ssh_hsk_algo: pattern=        - ssh_hsk_override: pattern=        - ssh_hsk_password: max_length=128 pattern=        - ssh_hsk: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    ssh_kex_algo: list[SshConfigSshKexAlgoEnum] = Field(default_factory=list, description="Select one or more SSH kex algorithms.")    
    ssh_enc_algo: list[SshConfigSshEncAlgoEnum] = Field(default_factory=list, description="Select one or more SSH ciphers.")    
    ssh_mac_algo: list[SshConfigSshMacAlgoEnum] = Field(default_factory=list, description="Select one or more SSH MAC algorithms.")    
    ssh_hsk_algo: list[SshConfigSshHskAlgoEnum] = Field(default_factory=list, description="Select one or more SSH hostkey algorithms.")    
    ssh_hsk_override: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable SSH host key override in SSH daemon.")    
    ssh_hsk_password: Any = Field(max_length=128, default=None, description="Password for ssh-hostkey.")    
    ssh_hsk: str = Field(description="Config SSH host key.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SshConfigModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SshConfigModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.813999Z
# ============================================================================


"""Type stubs for SANDBOX category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .cloud_regions import CloudRegions
    from .connection import Connection
    from .detect import Detect
    from .stats import Stats
    from .status import Status

__all__ = [
    "CloudRegions",
    "Connection",
    "Detect",
    "Stats",
    "Status",
    "Sandbox",
]


class Sandbox:
    """SANDBOX API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    cloud_regions: CloudRegions
    connection: Connection
    detect: Detect
    stats: Stats
    status: Status

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize sandbox category with HTTP client."""
        ...
